/*******************************************************************************
 * Example mongoose server
 *
 * main.c:  Setup, initialization and startup of the web server.
 ******************************************************************************/
#define MASTER
#define MAIN_C

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <signal.h>

#include "main.h"
#include "mongooseMgr.h"

/* === Global Variables === */
char *port = NULL;
char *documentRoot = NULL;

/**
 * Parse command line arguments.
 * Can't use Debug library here - it isn't initialized yet.
 */
static void 
parseArgs(int argc, char **argv)
{
    char opt;

    /* Suppress error messages from getopt_long() for unrecognized options. */
    opterr = 0;

    /* Parse the command line. */
    while ((opt = getopt(argc, argv, "p:")) != -1)
    {
        switch (opt)
        {
            /* -p <port> */
            case 'p':
                port = optarg;
                break;

            default:
                printf(USAGE);
                exit(0);
                break;
        }
    }

    /* Make sure port was specified. */
    if ( port == NULL )
    {
        printf(USAGE);
        exit(EXIT_FAILURE);
    }
}

/**
 * Global initialization at program startup.
 * Returns true (1) if setup completes successfully.
 * Returns false (0) if setup fails.
 */
static int
setup()
{
    char        *ptr;
    struct stat stat_buf;

    /* Set the document root of the web server. */
    ptr = getcwd(NULL, 0);
    documentRoot = (char *)malloc(strlen(ptr) + strlen(S_DOCROOT) + 1);
    memset(documentRoot, 0, strlen(ptr) + strlen(S_DOCROOT) + 1);
    strcpy(documentRoot, ptr);
    documentRoot = strcat(documentRoot, S_DOCROOT);

    /* Make sure the document root exists. If not, create it. */
    errno = 0;
    if ( stat (documentRoot, &stat_buf) != 0 )
    {
        if ( mkdir (documentRoot, 0700) != 0 )
            return(0);
    }

    /* Start mongoose. */
    mongooseMgrInit();
    return 1;
}

int 
main(int argc, char **argv)
{
    struct stat     stat_buf;
    char *progName = argv[0];

    /* Must parse the command line args first.*/
    parseArgs(argc, argv);

    /* Setup the web server and curl library. */
    if ( !setup() )
        return -1;

    /* Loop forever - mongoose will handle processing from here. */
    for (;;)
        sleep(1);
    return 0;
}
